@@ -54,7 +54,7 @@
 #define LIKELY(x) __builtin_expect ((x), 1)
 #define UNLIKELY(x) __builtin_expect ((x), 0)
 
-static inline void *
+__attribute__ ((malloc)) static inline void *
 xmalloc (size_t size)
 {
   void *res = malloc (size);
@@ -63,7 +63,7 @@ xmalloc (size_t size)
   return res;
 }
 
-static inline void *
+__attribute__ ((malloc)) static inline void *
 xmalloc0 (size_t size)
 {
   void *res = calloc (1, size);
@@ -72,7 +72,7 @@ xmalloc0 (size_t size)
   return res;
 }
 
-static inline void *
+__attribute__ ((malloc)) static inline void *
 xrealloc (void *ptr, size_t size)
 {
   void *res = realloc (ptr, size);
@@ -99,7 +99,7 @@ cleanup_filep (FILE **f)
 static inline void
 cleanup_closep (void *p)
 {
-  int *pp = p;
+  int *pp = (int *) p;
   if (*pp >= 0)
     TEMP_FAILURE_RETRY (close (*pp));
 }
@@ -107,7 +107,7 @@ cleanup_closep (void *p)
 static inline void
 cleanup_pidp (void *p)
 {
-  pid_t *pp = p;
+  pid_t *pp = (pid_t *) p;
   if (*pp > 0)
     {
       TEMP_FAILURE_RETRY (kill (*pp, SIGKILL));
@@ -115,8 +115,7 @@ cleanup_pidp (void *p)
     }
 }
 
-struct
-    libcrun_fd_map
+struct libcrun_fd_map
 {
   size_t nfds;
   int fds[];
@@ -128,7 +127,7 @@ make_libcrun_fd_map (size_t len)
   struct libcrun_fd_map *ret;
   size_t i;
 
-  ret = xmalloc (sizeof (*ret) + sizeof (int) * len);
+  ret = (struct libcrun_fd_map *) xmalloc (sizeof (*ret) + sizeof (int) * len);
   ret->nfds = len;
   for (i = 0; i < len; i++)
     ret->fds[i] = -1;
@@ -205,7 +204,25 @@ consume_slashes (const char *t)
   return t;
 }
 
-int xasprintf (char **str, const char *fmt, ...);
+static inline bool
+path_is_slash_dev (const char *path)
+{
+  path = consume_slashes (path);
+
+  if (strncmp (path, "dev", 3))
+    return false;
+
+  path += 3;
+
+  /* Check there are only '/' left.  */
+  for (; *path; path++)
+    if (*path != '/')
+      return false;
+
+  return true;
+}
+
+int xasprintf (char **str, const char *fmt, ...) __attribute__ ((format (printf, 2, 3)));
 
 int crun_path_exists (const char *path, libcrun_error_t *err);
 
@@ -215,6 +232,8 @@ int write_file (const char *name, const void *data, size_t len, libcrun_error_t
 
 int write_file_at (int dirfd, const char *name, const void *data, size_t len, libcrun_error_t *err);
 
+int write_file_at_with_flags (int dirfd, int flags, mode_t mode, const char *name, const void *data, size_t len, libcrun_error_t *err);
+
 int crun_ensure_directory (const char *path, int mode, bool nofollow, libcrun_error_t *err);
 
 int crun_ensure_file (const char *path, int mode, bool nofollow, libcrun_error_t *err);
@@ -280,9 +299,9 @@ size_t format_default_id_mapping (char **ret, uid_t container_id, uid_t host_id,
 int run_process_with_stdin_timeout_envp (char *path, char **args, const char *cwd, int timeout, char **envp,
                                          char *stdin, size_t stdin_len, int out_fd, int err_fd, libcrun_error_t *err);
 
-int mark_for_close_fds_ge_than (int n, libcrun_error_t *err);
+int mark_or_close_fds_ge_than (int n, bool close_now, libcrun_error_t *err);
 
-void get_current_timestamp (char *out);
+void get_current_timestamp (char *out, size_t len);
 
 int set_blocking_fd (int fd, int blocking, libcrun_error_t *err);
 
@@ -290,7 +309,7 @@ int parse_json_file (yajl_val *out, const char *jsondata, struct parser_context
 
 int has_prefix (const char *str, const char *prefix);
 
-const char *find_executable (const char *executable_path, const char *cwd, const char *handler);
+char *find_executable (const char *executable_path, const char *cwd);
 
 int copy_recursive_fd_to_fd (int srcfd, int destfd, const char *srcname, const char *destname, libcrun_error_t *err);
 
@@ -300,6 +319,7 @@ int libcrun_initialize_selinux (libcrun_error_t *err);
 
 int libcrun_initialize_apparmor (libcrun_error_t *err);
 
+const char *find_annotation_map (json_map_string_string *annotations, const char *name);
 const char *find_annotation (libcrun_container_t *container, const char *name);
 
 int get_file_type_at (int dirfd, mode_t *mode, bool nofollow, const char *path);
@@ -315,12 +335,18 @@ int safe_openat (int dirfd, const char *rootfs, size_t rootfs_len, const char *p
 
 ssize_t safe_write (int fd, const void *buf, ssize_t count);
 
-int append_paths (char **out, libcrun_error_t *err, ...);
+int append_paths (char **out, libcrun_error_t *err, ...) __attribute__ ((sentinel));
 
-LIBCRUN_PUBLIC int libcrun_str2sig (const char *name);
+int str2sig (const char *name);
 
 int base64_decode (const char *iptr, size_t isize, char *optr, size_t osize, size_t *nbytes);
 int has_suffix (const char *source, const char *suffix);
 char *str_join_array (int offset, size_t size, char *const array[], const char *joint);
 
+static inline bool
+is_empty_string (const char *s)
+{
+  return s == NULL || s[0] == '\0';
+}
+
 #endif
